/*
 * Created on Oct 13, 2003
 *
 */
import kareltherobot.*;

import java.awt.Color;

/** An abstract superclass that defines the basic actions of a Turing Machine.
 * To use this, extend it and implement the state methods (including the 
 * required state1()) according to the template shown with state1 (below)
 * @author jbergin
 *
 */
public abstract class TuringMachine extends AugmentedRobot
{

	/** Construct a Turing Machine at the origin with an infinite number
	 * of beepers in state1.
	 */
	public TuringMachine()
	{
		super(1, 1, East, infinity);
		state1();
	}

	/**Construct a Turing Machine at the origin with an infinite number
	 * of beepers and a badge, in state1.
	 * @param badge the color badge the robot will wear
	 */
	public TuringMachine(Color badge)
	{
		super(1, 1, East, infinity, badge);
		state1();
	}

	public void putBlank()
	{
			while(nextToABeeper())
					pickBeeper();
	}

	public void putOne()
	{
			putBlank();
			putBeeper();
	}

	public void moveLeft()
	{       
			faceWest();
			if (frontIsClear())
			{       move();
			}
			else
			{       turnRight();
					move();
			}
	}

	public void moveRight()
	{
			faceSouth();
			if(frontIsClear())
			{       move();
			}
			else
			{       turnLeft();
					move();
			}
	}
	
	public void finalState()
	{       turnOff();
	} 
	
	/** The initial Turing Machine state for every turing machine. Note
	 * that the constructor sends the newly created turing machine this message
	 * as part of construction.<p>
	 *   The template for this and every state method
	 * in your subclasses is:<br>
	 *  (1) you <b>may</b> send the message putBlank or putOne, but do this first. <br>
	 *  (2) you <b>may</b> send the message moveLeft or moveRight, but do this second<br>
	 *  (3) you now <b>must</b> send the message corresponding to some state 
	 *     (perhaps this same one or another, including finalState).<br> 
	 * <p> An example is:
	 * <code>
public void state1()
{       if(nextToABeeper())
        {       putBlank();
                moveRight();
                state3();
        }
        else
        {       moveLeft();
                state7();
        }
} 
</code>
	 * 
	 */
	public abstract void state1();
	
}
