package kareltherobot;

import java.awt.*;
import java.applet.*;
import java.awt.event.*;

import kareltherobot.Directions;
//import java.util.*;

public class PhilosopherApplet	extends Applet implements  Directions // Directions OR RobotTester is essential.
{	
	public void init()
	{	setSize(400, 400);
		add(worldCommands); // Just a flow layout. Could change, of course. 
		add(showWorld);
		add(tasker);
		add(wantBuilder);
		add(fromText);
		add(toText);
		add(reset);
		
		tasker.addActionListener (
			new ActionListener ()	
			{	public void actionPerformed(ActionEvent e)
				{	new Thread() // This thread is essential or the world won't update in an applet
					{	public void run()
						{ task();
						}	
					}.start();
				}
			}
		); // Wow -- double nested anonymous inner class. 
		
		wantBuilder.addActionListener(
			new ActionListener()
			{	public void actionPerformed(ActionEvent e)
				{	if (builder == null) 
						builder = new MiniBuilder(true);
					else ((MiniBuilder)builder).setVisible(true);	
				}
			}
		); 
		
		showWorld.addActionListener(
			new ActionListener()
			{	public void actionPerformed(ActionEvent e)
				{	World.setVisible(true);	
					World.showSpeedControl(true);	
				}
			}
		); 

		
		fromText.addActionListener(
			new ActionListener()
			{	public void actionPerformed(ActionEvent e)
				{	World.getWorld(worldCommands.getText()); //String -> World
					view.repaint();	
				}
			}
		);

		reset.addActionListener(
			new ActionListener()
			{	public void actionPerformed(ActionEvent e)
				{	World.reset();
					view.repaint();	
				}
			}
		);
		
		toText.addActionListener(
			new ActionListener()
			{	public void actionPerformed(ActionEvent e)
				{	String text = World.asText("\n"); // World -> String
					worldCommands.append(text);	
				}
			}
		);
		
		setVisible(true);
				
	}
	
	private Button tasker = new Button ("Perform Task");
	private TextArea worldCommands = new TextArea(20, 40); // A bit smaller is ok too.
	private RobotVision view = World.view(); // force the World class to load.
	private Button wantBuilder = new Button("World Builder");
	private WorldBuilderInterface builder = null;
	private Button reset = new Button("Clear World");
	private Button fromText = new Button("Text->World");
	private Button toText = new Button("World->Text");
	private Button showWorld = new Button("Show World");
	
/// Everything above is scaffolding
/// Everything below is what the user writes and includes

/// The class is needed but doesn't need to be inner to the applet. 
/// Being able to write classes is essential to the concept

class AugmentedRobot extends Robot 
{	public AugmentedRobot(int street, int avenue, Direction direction, int beepers, Color badge)
	{	super(street, avenue, direction, beepers, badge);
	}
	
	void  turnAround ()
	{	turnLeft();
		turnLeft();
	}

	void  turnRight ()
	{	turnLeft();
		turnLeft();
		turnLeft();
	}

	void  backUp ()
	{	turnAround();
		move();
		turnAround();
	}

	boolean  leftIsClear()
	{	turnLeft();
		if( frontIsClear() )
		{	turnRight();
			return true;
		}
		else
		{	turnRight();
			return false;
		}
	}

	boolean  rightIsClear()
	{	turnRight();
		if( frontIsClear() )
		{	turnLeft();
			return true;
		}
		else
		{	turnLeft();
			return false;
		}
	}

	void  faceNorth()
	{	while( ! facingNorth() )
		{	turnLeft(); 
		}
	}

	void  faceEast()
	{	while( ! facingEast() )
		{	turnLeft(); 
		}
	}

	void  faceSouth()
	{	while( ! facingSouth() )
		{	turnLeft(); 
		}
	}

	void  faceWest()
	{	while( ! facingWest() )
		{	turnLeft(); 
		}
	}

}

public class CommunicatingRobot extends AugmentedRobot
{	public CommunicatingRobot(int street, int avenue,  Direction direction, int beepers, Color badge)
	{	super(street, avenue, direction, beepers, badge);
		World.setupThread(this);
	}
	
	/** This method supposes some other robot will place a beeper on the current corner and
	* leave it long enough for this to awaken. 
	* BEWARE. When this returns the beeper that was seen, might have already been removed
	* by another robot. 
	*/
	public void waitForBeeper()
	{	while(!nextToABeeper()) Thread.yield(); // nothing. 
	}
	
	/** This method supposes some other robot show up at this corner and wait for a while.
	* BEWARE. When this returns the robot that caused it to return might have already
	* moved. A robot might also visit this corner and then move, but its presence not be noted.
	*/
	public void waitForRobot()
	{	while(!nextToARobot()) Thread.yield(); // nothing. 
	}
	
	/** BEWARE. This method can fail if another robot places a beeper and then quickly
	* retrieves it. 
	*/
	public void pickBeeperWhenPresent()
	{	waitForBeeper();
		pickBeeper();
	}

}


public class Philosopher extends CommunicatingRobot 
{	public Philosopher(int street, int avenue,  Direction direction, Color badge)
	{	super(street, avenue, direction, 0, badge);
		if(World.delay() == 0) World.setDelay(1); // May not run well at fastest speed. 
	}
	
	public void run()
	{	while(true)
		{	think(d.roll());
			getForks();
			eat(d.roll());
			putForks();
		}
	}
	
	private void think(int time)
	{	for(int i = 0; i < time; ++i)
		{	backUp();
			move();
		}
	}
	
	private void eat(int time)
	{	for(int i = 0; i < time; ++i)
		{	move();
			backUp();
		}
	}
	
	private void getForks()
	{	turnLeft();
		move();
		while(! anyBeepersInBeeperBag())
		{	pickBeeperWhenPresent();
		}
		turnAround();
		move();putBeeper();
		move();
		while(! anyBeepersInBeeperBag())
		{	pickBeeperWhenPresent();
		}
		turnAround();
		move();
		putBeeper();
		turnRight();
		showState("Eat ");
	}
	
	private void putForks()
	{	pickBeeper();
		pickBeeper();
		turnLeft();
		move();
		putBeeper();
		turnAround();
		move();
		move();
		putBeeper();
		turnAround();
		move();
		turnRight();
		showState("Think ");
	}
	
	private Die d = new Die(4);
}

	public void task() // Naming this "task" isn't essential, though desirable if visible.
	{		
	World.placeBeepers(2, 2, 1);
	World.placeBeepers(2, 4, 1);
	World.placeBeepers(4, 2, 1);
	World.placeBeepers(4, 4, 1);
	Philosopher p1 = new Philosopher(2, 3, North, Color.red);
	Philosopher p2 = new Philosopher(3, 2, East, Color.magenta);
	Philosopher p3 = new Philosopher(4, 3, South, Color.blue);
	Philosopher p4 = new Philosopher(3, 4, West, Color.green);
	}

}


